<?php
/**
 * Firecall Admin Settings Class
 *
 * @package  Firecall\Admin
 * @version  1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Firecall_Admin_Settings', false ) ) :

	/**
	 * Firecall_Admin_Settings Class.
	 */
	class Firecall_Admin_Settings {

		/**
		 * Setting pages.
		 *
		 * @var array
		 */
		private static $settings = array();

		/**
		 * Error messages.
		 *
		 * @var array
		 */
		private static $errors = array();

		/**
		 * Update messages.
		 *
		 * @var array
		 */
		private static $messages = array();

		/**
		 * Include the settings page classes.
		 */
		public static function get_settings_pages() {
			if ( empty( self::$settings ) ) {
				$settings = array();
				include_once dirname( __FILE__ ) . '/settings/class-firecall-setting-page.php';
				$settings[] = include __DIR__ . '/settings/class-firecall-setting-general.php';
				$settings[] = include __DIR__ . '/settings/class-firecall-setting-agentsetting.php';
				$settings[] = include __DIR__ . '/settings/class-firecall-setting-callsetting.php';
				self::$settings = apply_filters( 'firecall_get_settings_pages', $settings );
			}
			return self::$settings;
		}

		/**
		 * Save the settings.
		 */ 
		public static function save() {
			global $current_tab;
			$current_tab = esc_attr((isset($_GET['tab']))?$_GET['tab']:'general');

			check_admin_referer( 'firecall-settings' );
			// Trigger actions.
			do_action( 'firecall_settings_save_' . $current_tab );
			do_action( 'firecall_update_options_' . $current_tab );
			do_action( 'firecall_update_options' );

			self::add_message( esc_html__( 'Your settings have been saved.', 'firecall' ) );
			// self::check_download_folder_protection();

			// Clear any unwanted data and flush rules.
			update_option( 'firecall_queue_flush_rewrite_rules', 'yes' );
			
			do_action( 'firecall_settings_saved' );
		}

		/**
		 * Add a message.
		 *
		 * @param string $text Message.
		 */
		public static function add_message( $text ) {
			self::$messages[] = $text;
		}

		/**
		 * Add an error.
		 *
		 * @param string $text Message.
		 */
		public static function add_error( $text ) {
			self::$errors[] = $text;
		}

		/**
		 * Output messages + errors.
		 */
		public static function show_messages() {
			if ( count( self::$errors ) > 0 ) {
				foreach ( self::$errors as $error ) {
					echo '<div id="message" class="error inline"><p><strong>' . esc_html__( $error, 'firecall' ) . '</strong></p></div>';
				}
			} elseif ( count( self::$messages ) > 0 ) {
				foreach ( self::$messages as $message ) {
					echo '<div id="message" class="updated inline"><p><strong>' . esc_html__( $message , 'firecall') . '</strong></p></div>';
				}
			}
		}

		/**
		 * Settings page.
		 *
		 * Handles the display of the main firecall settings page in admin.
		 */
		public static function output() { 
			global $current_section, $current_tab;

			$current_tab = (isset($_REQUEST['tab']))?$_REQUEST['tab'] : 'general';
			$current_tab = esc_attr($current_tab);			

			do_action( 'firecall_settings_start' );

			// Get tabs for the settings page.
			$tabs = apply_filters( 'firecall_settings_tabs_array', array());

			include dirname( __FILE__ ) . '/views/firecall-admin-page.php';
		}

		/**
		 * Get a setting from the settings API.
		 *
		 * @param string $option_name Option name.
		 * @param mixed  $default     Default value.
		 * @return mixed
		 */
		public static function get_option( $option_name, $default = '' ) {
			if ( ! $option_name ) {
				return $default;
			}

			// Array value.
			if ( strstr( $option_name, '[' ) ) {

				parse_str( $option_name, $option_array );

				// Option name is first key.
				$option_name = current( array_keys( $option_array ) );

				// Get value.
				$option_values = get_option( $option_name, '' );

				$key = key( $option_array[ $option_name ] );

				if ( isset( $option_values[ $key ] ) ) {
					$option_value = $option_values[ $key ];
				} else {
					$option_value = null;
				}
			} else {
				// Single value.
				$option_value = get_option( $option_name, null );
			}

			if ( is_array( $option_value ) ) {
				$option_value = wp_unslash( $option_value );
			} elseif ( ! is_null( $option_value ) ) {
				$option_value = stripslashes( $option_value );
			}

			return ( null === $option_value ) ? $default : $option_value;
		}

		/**
		 * Output admin fields.
		 *
		 * Loops through the firecall options array and outputs each field.
		 *
		 * @param array[] $options Opens array to output.
		 */
		public static function output_fields( $options ) {
			foreach ( $options as $value ) {
				if ( ! isset( $value['type'] ) ) {
					continue;
				}
				if ( ! isset( $value['id'] ) ) {
					$value['id'] = '';
				}
				if ( ! isset( $value['title'] ) ) {
					$value['title'] = isset( $value['name'] ) ? $value['name'] : '';
				}
				if ( ! isset( $value['class'] ) ) {
					$value['class'] = '';
				}
				if ( ! isset( $value['css'] ) ) {
					$value['css'] = '';
				}
				if ( ! isset( $value['default'] ) ) {
					$value['default'] = '';
				}
				if ( ! isset( $value['desc'] ) ) {
					$value['desc'] = '';
				}
				if ( ! isset( $value['desc_tip'] ) ) {
					$value['desc_tip'] = true;
				}
				if ( ! isset( $value['placeholder'] ) ) {
					$value['placeholder'] = '';
				}
				if ( ! isset( $value['suffix'] ) ) {
					$value['suffix'] = '';
				}
				if ( ! isset( $value['value'] ) ) {
					$value['value'] = self::get_option( $value['id'], $value['default'] );
				}

				// Custom attribute handling.
				$custom_attributes = array();

				if ( ! empty( $value['custom_attributes'] ) && is_array( $value['custom_attributes'] ) ) {
					foreach ( $value['custom_attributes'] as $attribute => $attribute_value ) {
						$custom_attributes[] = esc_attr( $attribute ) . '="' . esc_attr( $attribute_value ) . '"';
					}
				}

				// Description handling.
				$field_description = self::get_field_description( $value );
				$description       = $field_description['description'];
				$tooltip_html      = wp_kses_post($field_description['tooltip_html'], 'firecall');

				// Switch based on type.
				switch ( $value['type'] ) {

					// Section Titles.
					case 'title':
					echo '<div class="firecall_title_wrap accordion">';
					echo '<div class="heading_wrap '.esc_attr($value['class']).'">';
					if ( ! empty( $value['title'] ) ) {
						echo '<h2>' . esc_html__( $value['title'], 'firecall' ) . '</h2>';
					}
					if ( ! empty( $value['desc'] ) ) {
						echo '<div id="' . esc_attr( sanitize_title( $value['id'] ) ) . '-description">';
						echo wp_kses_post( wpautop( wptexturize( $value['desc'] ) ) );
						echo '</div>';
					}
					echo '</div>';
					
					echo '</div>';
					$table_class  = isset($value['table_class'])?esc_attr($value['table_class']):'';
					echo '<table class="form-table panel '.$table_class.'" id="table-'.esc_attr($value['id']).'">' . "\n\n";
					if ( ! empty( $value['id'] ) ) {
						do_action( 'firecall_settings_' . sanitize_title( $value['id'] ) );
					}
					break;

					// Section Ends.
					case 'sectionend':
					if ( ! empty( $value['id'] ) ) {
						do_action( 'firecall_settings_' . sanitize_title( $value['id'] ) . '_end' );
					}
					echo '</table>';
					if ( ! empty( $value['id'] ) ) {
						do_action( 'firecall_settings_' . sanitize_title( $value['id'] ) . '_after' );
					}
					break;

					// Standard text inputs and subtypes like 'number'.
					case 'preview_design':
					?>
					<tr valign="top">
						<th scope="row" class="titledesc">
							<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html__( $value['title'], 'firecall' ); ?></label>
						</th>
						<td class="forminp forminp-<?php echo esc_attr( sanitize_title( $value['type'] ) ); ?>">
							<fieldset>
								<div class="<?php echo esc_attr( $value['class'] ); ?>" id="<?php echo esc_attr( $value['id'] ); ?>">
									<?php 
									$select_id =  $value['id']?str_replace("_preview","",$value['id']):'';
									$option_selected_value = esc_attr(get_option($select_id));
									$image_name = 'agent-layout-'.$option_selected_value.'.png';

									$previewimg = FIRECALL_PLUGIN_FILE.'images/'.$image_name;
									$previewimg = $previewimg?esc_url($previewimg):'';
									echo '<img src="'.esc_url($previewimg).'" class="preview_image">';
									?>


								</div>
							</fieldset>
						</td>
					</tr>
					<?php
					break;
					case 'number':
					case 'text':
					case 'url':
					case 'tel':
					case 'email':
					$option_value = $value['value'];

				?><tr valign="top">
					<th scope="row" class="titledesc">
						<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html__( $value['title'], 'firecall' ); ?> </label>
					</th>
					<td class="forminp forminp-<?php echo esc_attr( sanitize_title( $value['type'] ) ); ?>">
						<input
						name="<?php echo esc_attr( $value['id'] ); ?>"
						id="<?php echo esc_attr( $value['id'] ); ?>"
						type="<?php echo esc_attr( $value['type'] ); ?>"
						style="<?php echo esc_attr( $value['css'] ); ?>"
						value="<?php echo esc_attr( $option_value ); ?>"
						class="<?php echo esc_attr( $value['class'] ); ?>"
						placeholder="<?php echo esc_attr( $value['placeholder'] ); ?>"
						<?php echo implode( ' ', $custom_attributes ); // WPCS: XSS ok. ?>
						/><?php echo esc_html__( $value['suffix'], 'firecall' ); ?> <?php echo wp_kses_post($description, 'firecall'); // WPCS: XSS ok. ?>
					</td>
				</tr>
				<?php
				break;

				case 'color':
				$option_value = $value['value'];
				?>
				<tr valign="top">
					<th scope="row" class="titledesc">
						<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html__( $value['title'], 'firecall' ); ?> </label>
					</th>
					<td class="forminp forminp-<?php echo esc_attr( sanitize_title( $value['type'] ) ); ?>">
						<input
						name="<?php echo esc_attr( $value['id'] ); ?>"
						id="<?php echo esc_attr( $value['id'] ); ?>"
						type="text"
						style="<?php echo esc_attr( $value['css'] ); ?>"
						value="<?php echo esc_attr( $option_value ); ?>"
						class="color_pallete <?php echo esc_attr( $value['class'] ); ?>"
						<?php echo implode( ' ', $custom_attributes ); // WPCS: XSS ok. ?>
						/><?php echo esc_html__( $value['suffix'], 'firecall' ); ?> <?php echo wp_kses_post($description, 'firecall'); // WPCS: XSS ok. ?>
					</td>
				</tr>
				<?php
				break;

				case 'repeater':
				$lists = $value['value'];
				?>
				<div id="firecall_agent_list_wrap">
					<?php 
					if(isset($lists) && is_array($lists)){
						$li = 0;
						foreach($lists as $list){ 
							$firecall_ar_img = (isset($list['firecall_ar_image']) && !empty($list['firecall_ar_image']))?$list['firecall_ar_image']:FIRECALL_PLUGIN_FILE.'images/user-image.png';
							$firecall_ar_name = (isset($list['firecall_ar_name']))?$list['firecall_ar_name']:''; 
							$firecall_ar_position = (isset($list['firecall_ar_position']))?$list['firecall_ar_position']:'';
							$firecall_ar_description = (isset($list['firecall_ar_description']))?$list['firecall_ar_description']:'';
							$firecall_ar_telephone = (isset($list['firecall_ar_telephone']))?$list['firecall_ar_telephone']:'';
							?> 
							<div class="agent_list_item">
								<div class="agent_box_heading"><h2><?php esc_html_e('Agent Details', 'firecall'); ?></h2></div>
								<h2 class="remove_agent_title"><a class="removeagent"><?php esc_html_e('Remove Agent', 'firecall'); ?></a></h2>
								<div class="agent_image">
									<div class="agent_image_input_box">
										<h2><?php esc_html_e('Agent Image', 'firecall'); ?></h2>
										<img src="<?php echo esc_url($firecall_ar_img); ?>" class="agent_image"> 
										<input type="hidden" name="firecall_agent_list[<?php _e($li); ?>][firecall_ar_image]" class="regular-text agent_img_val" value="<?php echo esc_url($firecall_ar_img); ?>">
									</div>
									<div class="agent_upload_input_box">
										<input type="button" name="upload-btn" class="agent_img_upload" value="<?php esc_html_e('Upload Image', 'firecall'); ?>">
										<p class="fc_image_desc"><?php esc_html_e('* Image Dimension should be 60*60 pixel.', 'firecall'); ?></p>
									</div>
								</div>
								<div class="agent_box_content">
									<div class="agent_input_box">
										<h2><?php esc_html_e('Agent Name', 'firecall'); ?></h2>
										<input type="text" class="firecall_agent_list" size="<?php echo esc_attr(20); ?>" name="firecall_agent_list[<?php _e($li); ?>][firecall_ar_name]" value="<?php esc_html_e($firecall_ar_name, 'firecall'); ?>" placeholder="<?php esc_html_e('Name', 'firecall'); ?>" />
									</div>
									
									<div class="agent_input_box">
										<h2><?php esc_html_e('Agent Position', 'firecall'); ?></h2>
										<input type="text" name="firecall_agent_list[<?php _e($li); ?>][firecall_ar_position]" value="<?php esc_html_e($firecall_ar_position, 'firecall'); ?>" placeholder="<?php esc_html_e('Position', 'firecall'); ?>" />
									</div>

									<div class="agent_input_box">
										<h2><?php esc_html_e('Agent Description', 'firecall'); ?></h2>
										<input type="textarea" name="firecall_agent_list[<?php _e($li); ?>][firecall_ar_description]" value="<?php esc_html_e($firecall_ar_description, 'firecall'); ?>" placeholder="<?php esc_html_e('Description', 'firecall'); ?>" />
									</div>
									
									<div class="agent_input_box">
										<h2><?php esc_html_e('Agent Contact Number', 'firecall'); ?></h2>
										<input type="tel" name="firecall_agent_list[<?php _e($li); ?>][firecall_ar_telephone]" value="<?php echo esc_attr($firecall_ar_telephone); ?>" placeholder="<?php esc_html_e('Number', 'firecall'); ?>" />
									</div>
								</div>
								
							</div>
							<?php
							$li++;
						}
					}
					?>
				</div>
				<div>
					<h2 class="add_agent_title"><a href="#" id="addagent"><?php esc_html_e('Add Agent', 'firecall'); ?></a></h2>
				</div>
				
				
				<?php

				break;

				case 'texteditor':
				$option_value = $value['value'];

				?>
				<tr valign="top">
					<th scope="row" class="titledesc">
						<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html__( $value['title'], 'firecall' ); ?> </label>
					</th>
					<td class="forminp forminp-<?php echo esc_attr( sanitize_title( $value['type'] ) ); ?>">
						<?php 
						wp_editor( $option_value, esc_attr( $value['id'] ), $settings = array('media_buttons'=>false, 'quicktags' => false, 'tinymce' => array(
							'toolbar1' => 'bold, italic, underline',
							'toolbar2'=>false
						),'textarea_rows'=> $value['rows']) ); ?>
					</td>
				</tr>
				<?php
				break;

				// Select boxes.
				case 'select':
				case 'multiselect':
				$option_value = $value['value'];	

				?>
				<tr valign="top">
					<th scope="row" class="titledesc">
						<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html__( $value['title'] , 'firecall'); ?> </label>
					</th>
					<td class="forminp forminp-<?php echo esc_attr( sanitize_title( $value['type'] ) ); ?>">
						<select
						name="<?php echo esc_attr( $value['id'] ); ?><?php echo ( 'multiselect' === $value['type'] ) ? '[]' : ''; ?>"
						id="<?php echo esc_attr( $value['id'] ); ?>"
						style="<?php echo esc_attr( $value['css'] ); ?>"
						class="<?php echo esc_attr( $value['class'] ); ?>"
						<?php echo implode( ' ', $custom_attributes ); // WPCS: XSS ok. ?>
						<?php echo 'multiselect' === esc_attr($value['type']) ? 'multiple="multiple"' : ''; ?>
						>
						<?php
						foreach ( $value['options'] as $key => $val ) {
							?>
							<option data-img="<?php echo (isset($val[1]))?$val[1]:''; ?>" value="<?php echo esc_attr( $key ); ?>"
								<?php

								if ( is_array( $option_value ) ) {
									selected( in_array( (string) $key, $option_value, true ), true );
								} else {
									selected( $option_value, (string) $key );
								}

								?>
								><?php echo is_array($val)? $val[0] : esc_html__( $val , 'firecall'); ?></option>
								<?php
							}
							?>
							</select> <?php echo wp_kses_post($description, 'firecall'); // WPCS: XSS ok. ?>
						</td>
					</tr>
					<?php
					break;

					case 'fcradio':
					$option_value = $value['value'];	
					?>
					<tr valign="top">
						<th scope="row" class="titledesc">
							<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html__( $value['title'] , 'firecall'); ?> </label>
						</th>
						<td class="forminp forminp-<?php echo esc_attr( sanitize_title( $value['type'] ) ); ?>">
							<div id="<?php echo esc_attr( $value['id'] ); ?>">

								<?php
								$ii = 1;
								foreach($value['options'] as $op_key => $op_value){
									$firstcheck = '';
									$checked = ($op_key === $option_value)?'checked':'';
									
									if(empty($checked)){
										$firstcheck = esc_attr('checked');
									}
									$trimedkey = $op_key?strtolower(str_replace(' ','_',$op_key)):'';
									?>
									<div class="radio_list_item_wrap">
										<input type="radio" id="<?php echo esc_attr( $trimedkey ); ?>" name="<?php echo esc_attr( $value['id'] ); ?>" value="<?php echo esc_attr( $op_key ); ?>" <?php echo esc_attr($checked); ?> <?php echo ($ii === 1)?$firstcheck:''; ?> >
										<label for="<?php echo esc_attr($trimedkey); ?>"><?php echo wp_kses_post( $op_value ); ?></label>
									</div>
									<?php
									$ii++;
								} ?>

							</div>
						</td>
					</tr>
					<?php
					break;

					case 'switchbox':
					$option_value = esc_attr($value['value']);
					?>
					<tr valign="top">
						<th scope="row" class="titledesc">
							<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html__( $value['title'], 'firecall' ); ?></label>
						</th>
						<td class="forminp forminp-<?php echo esc_attr( sanitize_title( $value['type'] ) ); ?>">
							<fieldset>


								<div class="switch-field">
									<input 
									type="radio" 
									id="<?php echo esc_attr( $value['id'] ); ?>-unable"
									class="<?php echo esc_attr( $value['class'] ); ?>  switch-on"
									name="<?php echo esc_attr( $value['id'] ); ?>" 
									style="<?php echo esc_attr( $value['css'] ); ?>"
									<?php echo implode( ' ', $custom_attributes ); // WPCS: XSS ok. ?>
									value="Unable" <?php echo ($option_value === 'disable') ? '' :esc_attr('checked'); ?>/>

									<label for="<?php echo esc_attr( $value['id'] ); ?>-unable" class="left-check"><?php echo isset($value['unable'])?$value['unable']:esc_html__('On'); ?></label>

									<input 
									type="radio"
									id="<?php echo esc_attr( $value['id'] ); ?>-disable"
									class="<?php echo esc_attr( $value['class'] ); ?> switch-off"
									name="<?php echo esc_attr( $value['id'] ); ?>"
									style="<?php echo esc_attr( $value['css'] ); ?>"
									<?php echo implode( ' ', $custom_attributes ); // WPCS: XSS ok. ?>
									value="Disable" <?php echo ($option_value === 'disable') ? 'checked' :''; ?>/>
									<label for="<?php echo esc_attr( $value['id'] ); ?>-disable" class="right-check"><?php echo isset($value['disable'])?$value['disable']:esc_html__('Off'); ?></label> 
									<div id="flap"><span class="content"></span></div>
								</div>
								<?php echo $description; // WPCS: XSS ok. ?>
							</fieldset>
						</td>
					</tr>
					<?php
					break;
					case 'file':
					$option_value = esc_attr($value['value']);

					?>
					<tr valign="top">
						<th scope="row" class="titledesc">
							<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html__( $value['title'] , 'firecall'); ?></label>
						</th>
						<td class="forminp forminp-<?php echo esc_attr( sanitize_title( $value['type'] ) ); ?>">
							<fieldset>
								<?php 

								if ($option_value == '') {
									$remove_escaped = 'style=display:none;';
									$upload_escaped = false;
								} else {
									$remove_escaped = '';
									$upload_escaped = 'style=display:none;';
								}

								?>
								<div class="file-upload-field">
									<input type="text"
									value="<?php echo esc_attr($option_value); ?>"
									name="<?php echo esc_attr( $value['id'] ); ?>"
									class="<?php echo esc_attr( $value['class'] ); ?>" />
									<a href="javascript:void(0);" data-choose="<?php esc_html_e('Choose a File', 'firecall'); ?>" data-update="<?php esc_html_e('Select File', 'firecall'); ?>" class="firecall-upload" <?php echo esc_attr($upload_escaped); ?>><span></span><?php echo esc_html__('Browse', 'firecall'); ?></a>
									<a href="javascript:void(0);" class="firecall-upload-remove" <?php echo esc_attr($remove_escaped); ?>><?php echo esc_html__('Remove Upload', 'firecall'); ?></a>
								</div>
							</fieldset>
						</td>
					</tr>
					<?php 
					break;

					case 'button':
					$option_value =  esc_attr($value['value']);
					?>
					<tr valign="top">
						<th scope="row" class="titledesc">
							<label for="<?php echo esc_attr( $value['id'] ); ?>"><?php echo esc_html__( $value['title'], 'firecall' ); ?> </label>
						</th>
						<td class="forminp forminp-<?php echo esc_attr( sanitize_title( $value['type'] ) ); ?>">
							<input
							name="<?php echo esc_attr( $value['id'] ); ?>"
							id="<?php echo esc_attr( $value['id'] ); ?>"
							type="<?php echo esc_attr( $value['type'] ); ?>"
							value="<?php echo esc_attr( $option_value ); ?>"
							class="<?php echo esc_attr( $value['class'] ); ?>"
							<?php echo implode( ' ', $custom_attributes ); // WPCS: XSS ok. ?>
							/><?php echo esc_html__( $value['suffix'], 'firecall' ); ?> <?php echo wp_kses_post($description, 'firecall'); // WPCS: XSS ok. ?>
							<div class="reminder_msg"></div>
						</td>
					</tr>
					<?php
					break;
					case 'divstart':
					?> <div class="<?php echo esc_attr( $value['class'] ); ?>" id="<?php echo esc_attr( $value['id'] ); ?>"> <?php
					break;
									// Default: run an action.
					default:
					do_action( 'firecall_admin_field_' . $value['type'], $value );
					break;
				}
			}
		}

		/**
		* Helper function to get the formatted description and tip HTML for a
		* given form field. Plugins can call this when implementing their own custom
		* settings types.
		*
		* @param  array $value The form field value array.
		* @return array The description and tip as a 2 element array.
		*/

		public static function get_field_description( $value ) {

			$description  = '';
			$tooltip_html = '';

			if ( true === $value['desc_tip'] ) {
				$tooltip_html = $value['desc'];
			} elseif ( ! empty( $value['desc_tip'] ) ) {
				$description  = $value['desc'];
				$tooltip_html = $value['desc_tip'];
			} elseif ( ! empty( $value['desc'] ) ) {
				$description = $value['desc'];
			}

			if ( $description && in_array( $value['type'], array( 'textarea', 'radio' ), true ) ) {
				$description = '<p style="margin-top:0">' . wp_kses_post( $description , 'firecall' ) . '</p>';
			} elseif ( $description && in_array( $value['type'], array( 'checkbox' ), true ) ) {
				$description = wp_kses_post( $description, 'firecall' );
			} elseif ( $description ) {
				$description = '<p class="description">' . wp_kses_post( $description , 'firecall') . '</p>';
			}

			if ( $tooltip_html && in_array( $value['type'], array( 'checkbox' ), true ) ) {
				$tooltip_html = '<p class="description">' . esc_html__($tooltip_html, 'firecall') . '</p>';
			}

			return array(
				'description'  => $description,
				'tooltip_html' => $tooltip_html,
			);
		}

	/**
	* Save admin fields.
	*
	* Loops through the firecall options array and outputs each field.
	*
	* @param array $options Options array to output.
	* @param array $data    Optional. Data to use for saving. Defaults to $_POST.
	* @return bool
	*/
	public static function save_fields( $options, $data = null ) {
		if ( is_null( $data ) ) {
		$data = $_POST;		 // WPCS: input var okay, CSRF ok.
	}
	if ( empty( $data ) ) {
		return false;
	}

	// Options to update will be stored here and saved later.
	$update_options   = array();
	$autoload_options = array();

	// Loop options and get values to save.
	foreach ( $options as $option ) {
		if ( ! isset( $option['id'] ) || ! isset( $option['type'] ) || ( isset( $option['is_option'] ) && false === $option['is_option'] ) ) {
			continue;
		}


	// Get posted value.
		if ( strstr( $option['id'], '[' ) ) {
			parse_str( $option['id'], $option_name_array );
			$option_name  = current( array_keys( $option_name_array ) );
			$setting_name = key( $option_name_array[ $option_name ] );
			$raw_value    = isset( $data[ $option_name ][ $setting_name ] ) ? wp_unslash( $data[ $option_name ][ $setting_name ] ) : null;
		} else {
			$option_name  = $option['id'];
			$setting_name = '';
			$raw_value    = isset( $data[ $option['id'] ] ) ? wp_unslash( $data[ $option['id'] ] ) : null;
		}

	// Format the value based on option type.
		switch ( $option['type'] ) {

			case 'select':
			$allowed_values = empty( $option['options'] ) ? array() : array_map( 'strval', array_keys( $option['options'] ) );
			if ( empty( $option['default'] ) && empty( $allowed_values ) ) {
				$value = null;
				break;
			}
			$default = ( empty( $option['default'] ) ? $allowed_values[0] : $option['default'] );
			$value   = in_array( $raw_value, $allowed_values, true ) ? $raw_value : $default;
			break;
			default:
			$value =  $raw_value ;
			break;
		}

		/**
		* Fire an action when a certain 'type' of field is being saved.
		*
		* @deprecated 1.0.0 - doesn't allow manipulation of values!
		*/
		if ( has_action( 'firecall_update_option_' . sanitize_title( $option['type'] ) ) ) {
			wc_deprecated_function( 'The firecall_update_option_X action', '1.0.0', 'firecall_admin_settings_sanitize_option filter' );
			do_action( 'firecall_update_option_' . sanitize_title( $option['type'] ), $option );
			continue;
		}

		/**
		* Sanitize the value of an option.
		*
		* @since 1.0.0
		*/
		$value = apply_filters( 'firecall_admin_settings_sanitize_option', $value, $option, $raw_value );

		/**
		* Sanitize the value of an option by option name.
		*
		* @since 1.0.0
		*/
		$value = apply_filters( "firecall_admin_settings_sanitize_option_$option_name", $value, $option, $raw_value );

		if ( is_null( $value ) ) {
			continue;
		}

	// Check if option is an array and handle that differently to single values.
		if ( $option_name && $setting_name ) {
			if ( ! isset( $update_options[ $option_name ] ) ) {
				$update_options[ $option_name ] = get_option( $option_name, array() );
			}
			if ( ! is_array( $update_options[ $option_name ] ) ) {
				$update_options[ $option_name ] = array();
			}
			$update_options[ $option_name ][ $setting_name ] = $value;
		} else {
			$update_options[ $option_name ] = $value;
		}

		$autoload_options[ $option_name ] = isset( $option['autoload'] ) ? (bool) $option['autoload'] : true;

		/**
		* Fire an action before saved.
		*
		* @deprecated 1.0.0 - doesn't allow manipulation of values!
		*/
		do_action( 'firecall_update_option', $option );
	}

	// Save all options in our array.
	foreach ( $update_options as $name => $value ) {
		update_option( $name, $value, $autoload_options[ $name ] ? 'yes' : 'no' );
	}

	return true;
}

}

endif;